using FairyGUI;
using System.Collections.Generic;
using UnityEngine;
using XGame.Framework.Asyncs;

namespace XGame.Framework.FGUI
{
    /// <summary>
    /// ޸ͼ ο:https://zhuanlan.zhihu.com/p/689636245
    /// </summary>
    public partial class FguiModule : IGloaderHandle
    {
        public class LoadedAsset
        {
            public string addressable;
            public Sprite sprite;
            public NTexture nTexture;
        }

        /// <summary>
        /// ѼعԴ
        /// </summary>
        private Dictionary<string, LoadedAsset> _loadedAssetsMap = new Dictionary<string, LoadedAsset>();
        /// <summary>
        /// key:sprite.texture
        /// value:NTexture Spriteroot
        /// </summary>
        private Dictionary<Texture, NTexture> _textureRootMap = new Dictionary<Texture, NTexture>();

        IAsync IGloaderHandle.LoadNTexture(string addressableName)
        {
            if (_loadedAssetsMap.ContainsKey(addressableName))
            { // Ѿ
                var asyncGroup = new AsyncGroup();
                asyncGroup.End();
                return asyncGroup;
            }
            //TODO ҪʶSpriteTexture
            var loadAsync = _assetModule.LoadAsync<Sprite>(addressableName);
            loadAsync.On(_ =>
            {
                var result = loadAsync.GetResult();
                if (result == null)
                {
                    Log.Debug($"IGloaderHandle.LoadNTexture failed. addressable:{addressableName}");
                    return;
                }
                if (CreateNTexture(addressableName, result) == false)
                { //Ѿloader
                    _assetModule.Recycle(result);
                }
            });
            return loadAsync;
        }
        NTexture IGloaderHandle.GetNTexture(string addressableName)
        {
            if (_loadedAssetsMap.TryGetValue(addressableName, out var asset))
            {
                return asset.nTexture;
            }
            // ǼʧܣҲûмع
            Log.Debug($"ҲNTextureȷIGloaderHandle.LoadNTextureǷѾسɹ. Name:{addressableName}");
            return default;
        }

        private bool CreateNTexture(string addressable, Sprite sprite)
        {
            if (_loadedAssetsMap.ContainsKey(addressable))
            { // Ѿloader
                return false;
            }
            if (!_textureRootMap.TryGetValue(sprite.texture, out var root))
            {
                _textureRootMap[sprite.texture] = root = new NTexture(sprite.texture);
                root.destroyMethod = DestroyMethod.None;
            }
            Rect region = sprite.textureRect;
            region.y = sprite.texture.height - region.yMax;
            var nTexture = new NTexture(root, region, false);
            //עrelease¼GloaderDisposeʱֻᴥ NGraphics.Dispose() => _texture.ReleaseRef();
            nTexture.onRelease += OnReleaseNTexture;
            nTexture.destroyMethod = DestroyMethod.None;
            _loadedAssetsMap.Add(addressable, new LoadedAsset()
            {
                addressable = addressable,
                sprite = sprite,
                nTexture = nTexture,
            });
            return true;
        }
        private bool TryGetLoadedAsset(NTexture nTexture, out LoadedAsset asset)
        {
            foreach (var item in _loadedAssetsMap)
            {
                if (item.Value.nTexture == nTexture)
                {
                    asset = item.Value;
                    return true;
                }
            }
            asset = default;
            return false;
        }
        private void OnReleaseNTexture(NTexture nTexture)
        {
            if (TryGetLoadedAsset(nTexture, out var asset))
            {
                Log.Debug($"OnReleaseNTexture Addressable:{asset.addressable}");
                _loadedAssetsMap.Remove(asset.addressable);
                if (nTexture.root.refCount <= 0)
                {
                    _textureRootMap.Remove(asset.sprite.texture);
                }
                _assetModule.Recycle(asset.sprite);
            }
        }
        /// <summary>
        /// GloaderصԴ
        /// </summary>
        /// <param name="assetModule"></param>
        public void ClearGloaderAssets()
        {
            foreach (var item in _loadedAssetsMap)
            {
                _assetModule.Recycle(item.Value.sprite);
            }
            _loadedAssetsMap.Clear();
            _textureRootMap.Clear();
        }
    }
}